
//////////////////////////////////////////////////
//  SQLite Data Access Components
//  Copyright  2024 Devart. All right reserved.
//////////////////////////////////////////////////

{$I LiteDac.inc}

unit LiteConnectionEditor;

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages, Registry,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, Buttons, 
  {$IFDEF FPC}MaskEdit,{$ELSE}Mask,{$ENDIF}
{$IFDEF FPC}
  LResources,
{$ENDIF}
  CRTypes, CREditor,
  DBAccess, DAConnectionEditor,
  LiteAccess;

{$IFDEF MSWINDOWS}
const
  WM_SETDATABASETEXT = WM_USER + 1;
{$ENDIF}

type

  { TLiteConnectionEditorForm }

  TLiteConnectionEditorForm = class(TDAConnectionEditorForm)
    lbDatabase: TLabel;
    edDatabase: TComboBox;
    shOptions: TTabSheet;
    lbClientLibrary: TLabel;
    edClientLibrary: TEdit;
    btClientLibrary: TBitBtn;
    cbDirect: TCheckBox;
    lbAlgorithm: TLabel;
    edAlgorithm: TComboBox;
    procedure edDatabaseDropDown(Sender: TObject);
    procedure edDatabaseChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btClientLibraryClick(Sender: TObject);
    procedure edClientLibraryChange(Sender: TObject);
    procedure edPasswordChange(Sender: TObject);
    procedure cbDirectClick(Sender: TObject);
    procedure edAlgorithmChange(Sender: TObject);

  private
    FCurrItemText: string;

{$IFDEF MSWINDOWS}
    procedure WMSetDataBaseText(var Message: TMessage); message WM_SETDATABASETEXT;
{$ENDIF}

    procedure InternalSetDirect;
  protected
    function GetConnection: TLiteConnection;

    procedure ConnToControls; override;
    procedure FillInfo(InfoConnection: TCustomDAConnection); override;

    procedure DoInit; override;
    procedure DoSaveConnection; override;
    procedure DoFinish; override;

    function IsValidKeyValue(Value: string; Name: string): boolean;
    function GetApplicationKeyPath: string;
    procedure GetDatabaseList(List: TStrings);

    function BrowseForDatabase: string;
    procedure InternalSetDatabase;
    procedure DoSetDatabase;

  public
    property Connection: TLiteConnection read GetConnection;
  end;

implementation

{$IFNDEF FPC}
{$IFDEF IDE}
{$R *.dfm}
{$ELSE}
{$R LiteConnectionEditor.dfm}
{$ENDIF}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  MemData,
  LiteConsts, LiteCall;

{$IFDEF MSWINDOWS}
const
  LiteDacKey = '\SOFTWARE\Devart\LiteDAC\';
{$ENDIF}

{ TLiteConnectionEditorForm }

procedure TLiteConnectionEditorForm.DoFinish;
begin
  FLocalConnection.Disconnect;

  inherited;
end;

procedure TLiteConnectionEditorForm.DoInit;
begin
  inherited;

  lbVersion.Caption := LiteDACVersion + ' ';
  UpdateVersionPosition;

{$IFDEF BETA}
  lbEdition.Caption := 'Beta';
  lbEdition.Font.Color := clRed;
{$ELSE}
{$IFDEF RC}
  lbEdition.Caption := 'Release Candidate';
  lbEdition.Font.Color := clGreen;
{$ELSE}
{$IFDEF TRIAL}
  lbEdition.Caption := 'Trial Edition';
{$ELSE} //ORIGINAL
{$IFDEF STD}
  lbEdition.Caption := 'Standard Edition';
{$ELSE}
  lbEdition.Caption := 'Professional Edition';
{$ENDIF}
{$ENDIF}
{$ENDIF}
{$ENDIF}

  InternalSetDirect;

{$IFDEF LINUX}
  edDatabase.OnGetItems := edDatabaseDropDown;
{$ENDIF}
{$IFDEF DARWIN} 
  edDatabase.OnDropDown := edDatabaseDropDown;
{$ENDIF}

end;

procedure TLiteConnectionEditorForm.DoSaveConnection;
begin
  TLiteConnection(FConnection).Database := TLiteConnection(FLocalConnection).Database;
  TLiteConnection(FConnection).ClientLibrary := TLiteConnection(FLocalConnection).ClientLibrary;
  TLiteConnection(FConnection).EncryptionKey := TLiteConnection(FLocalConnection).EncryptionKey;
  TLiteConnection(FConnection).Options.Direct := TLiteConnection(FLocalConnection).Options.Direct;
  TLiteConnection(FConnection).Options.EncryptionAlgorithm := TLiteConnection(FLocalConnection).Options.EncryptionAlgorithm;

  inherited;
end;

procedure TLiteConnectionEditorForm.InternalSetDatabase;
{$IFDEF FPC}
var i: integer;
{$ENDIF}
begin
  edDatabase.SetFocus;
{$IFDEF FPC}
  i := edDatabase.Items.IndexOf(FCurrItemText);
  if i <> -1 then
    edDatabase.ItemIndex := i
  else begin
    edDatabase.Items.Add(FCurrItemText);
    edDatabase.ItemIndex := edDatabase.Items.Count - 1;
  end;
{$ELSE}
  edDatabase.Text := FCurrItemText;
{$ENDIF}
  edDatabase.SelectAll;

  DoSetDatabase;
end;

procedure TLiteConnectionEditorForm.DoSetDatabase;
begin
  if FInDoInit then
    Exit;

  if TLiteConnection(FLocalConnection).Database <> edDatabase.Text then
    try
      PerformDisconnect;
      TLiteConnection(FLocalConnection).Database := edDatabase.Text;
      Modified := True;

    finally
      ShowState;
    end;
end;

procedure TLiteConnectionEditorForm.FillInfo(InfoConnection: TCustomDAConnection);
var
  LiteInfoConnection: TLiteConnection;
  Query: TLiteQuery;
begin
  LiteInfoConnection := TLiteConnection(InfoConnection);

  meInfo.Lines.BeginUpdate;
  try
    meInfo.Lines.Clear;
    meInfo.Lines.Add('SQLite library version: ' + LiteInfoConnection.ClientLibraryVersion + #13#10);
    meInfo.Lines.Add('SQLite compilation options:');
    if LiteInfoConnection.Connected then begin
      Query := TLiteQuery.Create(nil);
      try
        Query.Connection := LiteInfoConnection;
        Query.SQL.Text := 'PRAGMA compile_options';
        Query.Open;
        while not Query.Eof do begin
          meInfo.Lines.Add(Query.Fields[0].AsString);
          Query.Next;
        end;
      finally
        Query.Close;
        Query.Free;
      end;
    end;
  finally
    meInfo.Lines.EndUpdate;
  end;
end;

function TLiteConnectionEditorForm.BrowseForDatabase: string;
var
  Dialog: TOpenDialog;
begin
  Dialog := TOpenDialog.Create(nil);
  try
  {$IFDEF MSWINDOWS}
    Dialog.Filter := 'SQLite Database Files (*.db3, *.db, *.sqlite)|*.db3;*.db;*.sqlite|All Files (*.*)|*.*';
  {$ELSE}
    Dialog.Filter := 'All Files (*)|*';
  {$ENDIF}
    if not TLiteConnection(FLocalConnection).Options.ForceCreateDatabase then
      Dialog.Options := Dialog.Options + [ofFileMustExist];
    if Dialog.Execute then
      Result := Dialog.FileName;
  finally
    Dialog.Free;
  end;
end;

procedure TLiteConnectionEditorForm.btClientLibraryClick(Sender: TObject);
var
  OpenDialog: TOpenDialog;
begin
  OpenDialog := TOpenDialog.Create(nil);
  try
  {$IFDEF MSWINDOWS}
    OpenDialog.Filter := 'Dynamic link library (*.dll)|*.dll|All Files (*.*)|*.*';
  {$ELSE}
    OpenDialog.Filter := 'All Files (*)|*';
  {$ENDIF}
    OpenDialog.Options := OpenDialog.Options + [ofFileMustExist];
    if OpenDialog.Execute then begin
      PerformDisconnect;
      edClientLibrary.SetFocus;
      edClientLibrary.Text := OpenDialog.FileName;
      edClientLibrary.SelectAll;
      edClientLibraryChange(nil);
    end;
  finally
    OpenDialog.Free;
  end;
end;

procedure TLiteConnectionEditorForm.ConnToControls;
var
  OldOnChange: TNotifyEvent;
begin
  inherited;

  TLiteConnection(FLocalConnection).ClientLibrary := Connection.ClientLibrary;

  edDatabase.Text := TLiteConnection(FLocalConnection).Database;
  edPassword.Text := TLiteConnection(FLocalConnection).EncryptionKey;

  OldOnChange := edClientLibrary.OnChange;
  edClientLibrary.OnChange := nil;
  try
    edClientLibrary.Text := TLiteConnection(FLocalConnection).ClientLibrary;
  finally
    edClientLibrary.OnChange := OldOnChange;
  end;

  OldOnChange := edAlgorithm.OnChange;
  edAlgorithm.OnChange := nil;
  try
    edAlgorithm.ItemIndex := Integer(TLiteConnection(FLocalConnection).Options.EncryptionAlgorithm);
  finally
    edAlgorithm.OnChange := OldOnChange;
  end;

  cbDirect.Checked := TLiteConnection(FLocalConnection).Options.Direct;
  InternalSetDirect;
end;

function TLiteConnectionEditorForm.GetApplicationKeyPath: string;
begin
{$IFDEF MSWINDOWS}
  Result := LiteDacKey + 'Connect\' + ApplicationTitle;
{$ELSE}  
  Result := '';
{$ENDIF}  
end;

function TLiteConnectionEditorForm.GetConnection: TLiteConnection;
begin
  Result := FConnection as TLiteConnection;
end;

procedure TLiteConnectionEditorForm.GetDatabaseList(List: TStrings);
{$IFDEF MSWINDOWS}
var
  ListKey, ServerKey: string;
  Registry: TRegistry;
  i: integer;
  KeyOpened: boolean;
{$ENDIF}  
begin
{$IFDEF MSWINDOWS}
  Registry := TRegistry.Create(KEY_READ OR KEY_WRITE);
  try
    ListKey := GetApplicationKeyPath;
    KeyOpened := Registry.OpenKey(ListKey, False);
    if not KeyOpened then begin
      ListKey := StringReplace(ListKey, 'Devart', 'CoreLab', [rfIgnoreCase]);
      KeyOpened := Registry.OpenKey(ListKey, False);
    end;
    if KeyOpened then begin
      List.Clear;
      ServerKey := 'Database';
      i := 1;
      while Registry.ValueExists(Format('%s %d', [ServerKey, i])) do begin
        List.Add(Registry.ReadString(Format('%s %d', [ServerKey, i])));
        Inc(i);
      end;
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;
{$ENDIF}  
end;

procedure TLiteConnectionEditorForm.InternalSetDirect;
begin
  edClientLibrary.Enabled := {$IFNDEF STD}not cbDirect.Checked{$ELSE}True{$ENDIF};
  btClientLibrary.Enabled := {$IFNDEF STD}not cbDirect.Checked{$ELSE}True{$ENDIF};
  lbClientLibrary.Enabled := {$IFNDEF STD}not cbDirect.Checked{$ELSE}True{$ENDIF};
  edAlgorithm.Enabled := {$IFNDEF STD}cbDirect.Checked{$ELSE}False{$ENDIF};
  lbAlgorithm.Enabled := {$IFNDEF STD}cbDirect.Checked{$ELSE}False{$ENDIF};
  cbDirect.Enabled := {$IFNDEF STD}True{$ELSE}False{$ENDIF};
end;

function TLiteConnectionEditorForm.IsValidKeyValue(Value,
  Name: string): boolean;
var
  p: Integer;
begin
  p := Pos(UpperCase(Name), UpperCase(Value));
  if p <> 0 then begin
    Inc(p, Length(Name) - 1);
    if p < Length(Value) then
      Inc(p);
    while (Byte(Value[p]) in [$30..$30+9, $20]) and (p <> Length(Value)) do
      Inc(p);
    Result := p = Length(Value);
  end
  else
    Result := False;
end;

{$IFDEF MSWINDOWS}
procedure TLiteConnectionEditorForm.WMSetDataBaseText(var Message: TMessage);
begin
  InternalSetDatabase;
end;
{$ENDIF}

procedure TLiteConnectionEditorForm.edDatabaseDropDown(Sender: TObject);
begin
{$IFDEF UNIX}
{$IFDEF LINUX}
    (Sender as TComboBox).OnGetItems := nil;
{$ENDIF}
{$IFDEF DARWIN} 
    (Sender as TComboBox).OnDropDown := nil;
{$ENDIF}
  try
{$ENDIF}
  StartWait;
  try
    FCurrItemText := edDatabase.Text;
    edDatabase.Items.Clear;
    GetDatabaseList(edDatabase.Items);
    edDatabase.Items.Add(SDBBrowse);
    if edDatabase.Items.Count < 20 then
      edDatabase.DropDownCount := edDatabase.Items.Count
    else
      edDatabase.DropDownCount := 20;
  finally
    StopWait;
  end;
{$IFDEF UNIX}
  finally
{$IFDEF LINUX}
    (Sender as TComboBox).OnGetItems := edDatabaseDropDown;
{$ENDIF}
{$IFDEF DARWIN} 
    (Sender as TComboBox).OnDropDown := edDatabaseDropDown;
{$ENDIF}
  end;
{$ENDIF}
end;

procedure TLiteConnectionEditorForm.edPasswordChange(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  if TLiteConnection(FLocalConnection).EncryptionKey <> edPassword.Text then
    try
      PerformDisconnect;
      TLiteConnection(FLocalConnection).EncryptionKey := edPassword.Text;
      Modified := True;
    finally
      ShowState;
    end;
end;

procedure TLiteConnectionEditorForm.edAlgorithmChange(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  if TLiteConnection(FLocalConnection).Options.EncryptionAlgorithm <> TLiteEncryptionAlgorithm(edAlgorithm.ItemIndex) then
    try
      PerformDisconnect;
      TLiteConnection(FLocalConnection).Options.EncryptionAlgorithm := TLiteEncryptionAlgorithm(edAlgorithm.ItemIndex);
      Modified := True;
    finally
      ShowState;
    end;
end;

procedure TLiteConnectionEditorForm.edClientLibraryChange(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  if TLiteConnection(FLocalConnection).ClientLibrary <> edClientLibrary.Text then
    try
      PerformDisconnect;
      TLiteConnection(FLocalConnection).ClientLibrary := edClientLibrary.Text;
      Modified := True;
    finally
      ShowState;
    end;
end;

procedure TLiteConnectionEditorForm.edDatabaseChange(Sender: TObject);
var
  DatabaseName: string;
begin
  if FInDoInit then
    Exit;

  if edDatabase.Text = SDBBrowse then begin
    DatabaseName := BrowseForDatabase;
    if DatabaseName <> '' then
      FCurrItemText := DatabaseName;
{$IFDEF MSWINDOWS}
    PostMessage(Handle, WM_SETDATABASETEXT, 0, 0);
{$ELSE}	
    InternalSetDatabase;
{$ENDIF}
  end
  else
    DoSetDatabase;
end;

procedure TLiteConnectionEditorForm.FormShow(Sender: TObject);
begin
  inherited;
  TDBAccessUtils.SetLockLoginPrompt(Connection, True);
end;

procedure TLiteConnectionEditorForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  inherited;
  if CanClose then
    TDBAccessUtils.SetLockLoginPrompt(Connection, False);
end;

procedure TLiteConnectionEditorForm.cbDirectClick(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  if TLiteConnection(FLocalConnection).Options.Direct <> cbDirect.Checked then
    try
      PerformDisconnect;
      TLiteConnection(FLocalConnection).Options.Direct := cbDirect.Checked;
      edAlgorithmChange(Sender);
      edPasswordChange(Sender);
      Modified := True;
    finally
      InternalSetDirect;
      ShowState;
    end;
end;

end.
